@@ -95,7 +95,7 @@ from collections import OrderedDict
 from datetime import time, date, datetime, timedelta
 
 __pkgname__ = 'python-crontab'
-__version__ = '2.0'
+__version__ = '2.0.2'
 
 ITEMREX = re.compile(r'^\s*([^@#\s]+)\s+([^@#\s]+)\s+([^@#\s]+)\s+([^@#\s]+)'
                      r'\s+([^@#\s]+)\s+([^#\n]*)(\s+#\s*([^\n]*)|$)')
@@ -139,6 +139,8 @@ SHELL = os.environ.get('SHELL', '/bin/sh')
 # The shell won't actually work on windows here, but
 # it should be updated later in the below conditional.
 
+# pylint: disable=W0622,invalid-name,too-many-public-methods
+# pylint: disable=function-redefined,too-many-instance-attributes
 current_user = lambda: None
 if not WINOS:
     import pwd
@@ -147,11 +149,9 @@ if not WINOS:
         return pwd.getpwuid(os.getuid())[0]
 
 if PY3:
-    # pylint: disable=W0622
     unicode = str
     basestring = str
 
-
 def open_pipe(cmd, *args, **flags):
     """Runs a program and orders the arguments for compatability.
 
@@ -301,7 +301,7 @@ class CronTab(object):
             # Add the entire crontab back to the user crontab
             if self.user:
                 proc = open_pipe(CRONCMD, path, **self.user_opt)
-                # XXX This could do with being cleaned up quite a bit
+                # This could do with being cleaned up quite a bit
                 proc.wait()
                 proc.stdout.close()
                 proc.stderr.close()
@@ -339,7 +339,7 @@ class CronTab(object):
         envs = self.env.items()
         env = ["%s=%s" % (key, _unicode(val)) for (key, val) in envs]
         crons = [unicode(cron) for cron in self.lines]
-        result = u'\n'.join(env) + u'\n'.join(crons)
+        result = u'\n'.join(env + crons)
         if result and result[-1] not in (u'\n', u'\r'):
             result += u'\n'
         return result
@@ -390,7 +390,7 @@ class CronTab(object):
                 yield cron.comment
                 returned.append(cron.comment)
 
-    def remove_all(self, **kwargs):
+    def remove_all(self, *args, **kwargs):
         """Removes all crons using the stated command OR that have the
         stated comment OR removes everything if no arguments specified.
 
@@ -398,6 +398,8 @@ class CronTab(object):
            comment - Remove all with this comment or ID
            time    - Remove all with this time code
         """
+        if args:
+            raise AttributeError("Invalid use: remove_all(command='cmd')")
         if 'command' in kwargs:
             return self.remove(*self.find_command(kwargs['command']))
         elif 'comment' in kwargs:
@@ -433,7 +435,9 @@ class CronTab(object):
         return "<Unattached %sCronTab>" % kind
 
     def __iter__(self):
-        return self.crons.__iter__()
+        """Return generator so we can track jobs after removal"""
+        for job in list(self.crons.__iter__()):
+            yield job
 
     def __getitem__(self, i):
         return self.crons[i]
@@ -618,8 +622,8 @@ class CronItem(object):
             shell = self.cron.env['SHELL']
         (out, err) = open_pipe(shell, '-c', self.command).communicate()
         if err:
-            LOG.error(err)
-        return out.strip()
+            LOG.error(err.decode("utf-8"))
+        return out.decode("utf-8").strip()
 
     def schedule(self, date_from=None):
         """Return a croniter schedule if available."""
@@ -755,15 +759,14 @@ class Every(object):
             raise ValueError("Invalid value '%s', outside 1 year" % self.unit)
         self.slices.setall('@yearly')
 
+
 class CronSlices(list):
     """Controls a list of five time 'slices' which reprisent:
         minute frequency, hour frequency, day of month frequency,
         month requency and finally day of the week frequency.
      """
     def __init__(self, *args):
-        super(CronSlices, self).__init__()
-        for info in S_INFO:
-            self.append(CronSlice(info))
+        super(CronSlices, self).__init__([CronSlice(info) for info in S_INFO])
         self.special = None
         if args and not self.setall(*args):
             raise ValueError("Can't set cron value to: %s" % str(args))
@@ -911,6 +914,8 @@ class Also(object):
 class CronSlice(object):
     """Cron slice object which shows a time pattern"""
     def __init__(self, info, value=None):
+        if isinstance(info, int):
+            info = S_INFO[info]
         self.min = info.get('min', None)
         self.max = info.get('max', None)
         self.name = info.get('name', None)
@@ -926,14 +931,13 @@ class CronSlice(object):
             return self.clear()
         for part in str(value).split(','):
             if part.find("/") > 0 or part.find("-") > 0 or part == '*':
-                self.parts.append(self.get_range(part))
-            else:
-                try:
-                    self.parts.append(self.parse_value(part))
-                except SundayError:
-                    self.parts.append(0)
-                except ValueError as err:
-                    raise ValueError('%s:%s/%s' % (str(err), self.name, part))
+                self.parts += self.get_range(part)
+                continue
+
+            try:
+                self.parts.append(self.parse_value(part, sunday=0))
+            except ValueError as err:
+                raise ValueError('%s:%s/%s' % (str(err), self.name, part))
 
     def render(self, resolve=False):
         """Return the slice rendered as a crontab.
@@ -961,7 +965,7 @@ class CronSlice(object):
         """Set the every X units value"""
         if not also:
             self.clear()
-        self.parts.append(self.get_range(int(n_value)))
+        self.parts += self.get_range(int(n_value))
         return self.parts[-1]
 
     def on(self, *n_value, **opts):
@@ -969,17 +973,14 @@ class CronSlice(object):
         if not opts.get('also', False):
             self.clear()
         for set_a in n_value:
-            try:
-                self.parts += self.parse_value(set_a),
-            except SundayError:
-                self.parts += 0,
+            self.parts += self.parse_value(set_a, sunday=0),
         return self.parts
 
     def during(self, vfrom, vto, also=False):
         """Set the During value, which sets a range"""
         if not also:
             self.clear()
-        self.parts.append(self.get_range(self.parse_value(vfrom), self.parse_value(vto)))
+        self.parts += self.get_range(str(vfrom) + '-' + str(vto))
         return self.parts[-1]
 
     @property
@@ -993,7 +994,10 @@ class CronSlice(object):
 
     def get_range(self, *vrange):
         """Return a cron range for this slice"""
-        return CronRange(self, *vrange)
+        ret = CronRange(self, *vrange)
+        if ret.dangling is not None:
+            return [ret.dangling, ret]
+        return [ret]
 
     def __iter__(self):
         """Return the entire element as an iterable"""
@@ -1014,7 +1018,7 @@ class CronSlice(object):
         """Returns the number of times this slice happens in it's range"""
         return len(list(self.__iter__()))
 
-    def parse_value(self, val):
+    def parse_value(self, val, sunday=None):
         """Parse the value of the cron slice and raise any errors needed"""
         if val == '>':
             val = self.max
@@ -1028,6 +1032,8 @@ class CronSlice(object):
             raise KeyError("No enumeration '%s' got '%s'" % (self.name, val))
 
         if self.max == 6 and int(out) == 7:
+            if sunday is not None:
+                return sunday
             raise SundayError("Detected Sunday as 7 instead of 0!")
 
         if int(out) < self.min or int(out) > self.max:
@@ -1035,10 +1041,6 @@ class CronSlice(object):
                 str(val), self.min, self.max, self.name))
         return out
 
-    def filter_value(self, val):
-        """Support wrapper for enumerations and check for range"""
-        return self.parse_value(val)
-
 
 def get_cronvalue(value, enums):
     """Returns a value as int (pass-through) or a special enum value"""
@@ -1089,6 +1091,8 @@ def _render(value, resolve=False):
 class CronRange(object):
     """A range between one value and another for a time range."""
     def __init__(self, vslice, *vrange):
+        # holds an extra dangling entry, for example sundays.
+        self.dangling = None
         self.slice = vslice
         self.cron = None
         self.seq = 1
@@ -1108,16 +1112,24 @@ class CronRange(object):
         """Parse a ranged value in a cronjob"""
         if value.count('/') == 1:
             value, seq = value.split('/')
-            self.seq = self.slice.filter_value(seq)
+            try:
+                self.seq = self.slice.parse_value(seq)
+            except SundayError:
+                self.seq = 1
+                value = "0-0"
             if self.seq < 1 or self.seq > self.slice.max - 1:
                 raise ValueError("Sequence can not be divided by zero or max")
         if value.count('-') == 1:
             vfrom, vto = value.split('-')
-            self.vfrom = self.slice.filter_value(vfrom)
+            self.vfrom = self.slice.parse_value(vfrom, sunday=0)
             try:
-                self.vto = self.slice.filter_value(vto)
+                self.vto = self.slice.parse_value(vto)
             except SundayError:
-                self.vto = 6
+                if self.vfrom == 1:
+                    self.vfrom = 0
+                else:
+                    self.dangling = 0
+                self.vto = self.slice.parse_value(vto, sunday=6)
         elif value == '*':
             self.all()
         else:
@@ -1132,7 +1144,10 @@ class CronRange(object):
         """Render the ranged value for a cronjob"""
         value = '*'
         if int(self.vfrom) > self.slice.min or int(self.vto) < self.slice.max:
-            value = _render_values([self.vfrom, self.vto], '-', resolve)
+            if self.vfrom == self.vto:
+                value = str(self.vfrom)
+            else:
+                value = _render_values([self.vfrom, self.vto], '-', resolve)
         if self.seq != 1:
             value += "/%d" % self.seq
         if value != '*' and SYSTEMV:
